@@ -16,6 +16,7 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
+import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.widgets.Display;
 
 import org.eclipse.draw2d.geometry.Rectangle;
@@ -33,7 +34,12 @@ public class DeferredUpdateManager
 protected class UpdateRequest
 	implements Runnable
 {
-	/**
+    
+	public UpdateRequest() {
+        super();
+    }
+
+    /**
 	 * Calls {@link DeferredUpdateManager#performUpdate()}.
 	 */
 	public void run() {
@@ -129,6 +135,22 @@ protected Graphics getGraphics(Rectangle region) {
 	return graphicsSource.getGraphics(region);
 }
 
+void paint(GC gc) {
+	SWTGraphics graphics = new SWTGraphics(gc);
+	if (!updating) {
+		/**
+		 * If a paint occurs not as part of an update, we should notify that the region
+		 * is being painted. Otherwise, notification already occurs in repairDamage().
+		 */
+		HashMap map = new HashMap();
+		Rectangle rect = graphics.getClip(new Rectangle());
+		map.put(root, rect);
+		firePainting(rect, map);
+	}
+	root.paint(graphics);
+	graphics.dispose();
+}
+
 /**
  * Performs the update.  Validates the invalid figures and then repaints the dirty
  * regions.
@@ -140,7 +162,7 @@ public synchronized void performUpdate() {
 		return;
 	updating = true;
 	try {
-		validateFigures();
+		performValidation();
 		updateQueued = false;
 		repairDamage();
 		if (afterUpdate != null) {
@@ -155,6 +177,25 @@ public synchronized void performUpdate() {
 	}
 }
 
+/**
+ * @see UpdateManager#performValidation()
+ */
+public void performValidation() {
+	if (invalidFigures.isEmpty())
+		return;
+	try {
+		IFigure fig;
+		fireValidating();
+		for (int i = 0; i < invalidFigures.size(); i++) {
+			fig = (IFigure) invalidFigures.get(i);
+			invalidFigures.set(i, null);
+			fig.validate();
+		}
+	} finally {
+		invalidFigures.clear();
+	}
+}
+
 /**
  * Adds the given exposed region to the update queue and then performs the update.
  * 
@@ -171,16 +212,25 @@ public synchronized void performUpdate(Rectangle exposed) {
  */
 protected void queueWork() {
 	if (!updateQueued) {
-		Display.getCurrent().asyncExec(new UpdateRequest());
+		sendUpdateRequest();
 		updateQueued = true;
 	}
 }
 
+/**
+ * Fires the <code>UpdateRequest</code> to the current display asynchronously.
+ * @since 3.2
+ */
+protected void sendUpdateRequest() {
+    Display.getCurrent().asyncExec(new UpdateRequest());
+}
+
 /**
  * Releases the graphics object, which causes the GraphicsSource to flush.
  * @param graphics the graphics object
  */
 protected void releaseGraphics(Graphics graphics) {
+	graphics.dispose();
 	graphicsSource.flushGraphics(damage);
 }
 
@@ -209,7 +259,6 @@ protected void repairDamage() {
 			damage = new Rectangle(contribution);
 		else
 			damage.union(contribution);
-//		keys.remove();
 	}
 
 	if (!dirtyRegions.isEmpty()) {
@@ -256,23 +305,11 @@ public void setRoot(IFigure figure) {
 }
 
 /**
- * Validates the invalid figures on the update queue and calls 
+ * Validates all invalid figures on the update queue and calls 
  * {@link UpdateManager#fireValidating()} unless there are no invalid figures.
  */
 protected void validateFigures() {
-	if (invalidFigures.isEmpty())
-		return;
-	try {
-		IFigure fig;
-		fireValidating();
-		for (int i = 0; i < invalidFigures.size(); i++) {
-			fig = (IFigure) invalidFigures.get(i);
-			invalidFigures.set(i, null);
-			fig.validate();
-		}
-	} finally {
-		invalidFigures.clear();
-	}
+	performValidation();
 }
 
 }
