@@ -151,19 +151,6 @@ private void captureSourceDimensions() {
 		if (child == getSourceEditPart())
 			sourceRectangle = bounds;
 	}
-	if (sourceRectangle == null) {
-		/*
-		 * @TODO:Pratik    Check to see if this ever happens.  I.e., if the operation
-		 * set does not include the source edit part.
-		 */
-		IFigure figure = ((GraphicalEditPart)getSourceEditPart()).getFigure();
-		if (figure instanceof HandleBounds)
-			sourceRectangle = new PrecisionRectangle(
-					((HandleBounds)figure).getHandleBounds());
-		else
-			sourceRectangle = new PrecisionRectangle(figure.getBounds());
-		figure.translateToAbsolute(sourceRectangle);
-	}	
 }
 
 /**
@@ -175,12 +162,17 @@ private void captureSourceDimensions() {
  * @see org.eclipse.gef.tools.AbstractTool#createOperationSet()
  */
 protected List createOperationSet() {
-	List list = ToolUtilities.getSelectionWithoutDependants(
-		getCurrentViewer());
-	ToolUtilities.filterEditPartsUnderstanding(list, getTargetRequest());
-	return list;
+    if (getCurrentViewer() != null) {
+       List list = ToolUtilities.getSelectionWithoutDependants(
+               getCurrentViewer());
+       ToolUtilities.filterEditPartsUnderstanding(list, getTargetRequest());
+       return list;
+    }
+
+    return new ArrayList();
 }
 
+
 /**
  * Creates a {@link ChangeBoundsRequest}.  By default, the type is
  * {@link RequestConstants#REQ_MOVE}.  Later on when the edit parts are asked to
@@ -522,7 +514,7 @@ protected void setTargetEditPart(EditPart editpart) {
 		return;
 	super.setTargetEditPart(editpart);
 	snapToHelper = null;
-	if (getTargetEditPart() != null)
+	if (getTargetEditPart() != null && getOperationSet().size() > 0)
 		snapToHelper = (SnapToHelper)getTargetEditPart().getAdapter(SnapToHelper.class);
 }
 
